function [ratios]=AnalyzeResults( Results, Notes, varargin)
% This function analyzes a cell array of results that are each computed by
% AnalyzeClip.m. It produces some top-level summary plots.

p=inputParser;

p.addRequired('Results');
p.addRequired('Notes');


p.addParamValue('DEBUG', false); %Debug data if true

p.parse(Results, Notes, varargin{:});
Args=p.Results;

ClipNum=[]; %The clip number (i.e., equal to numel of Results)
PeakNum=[]; %For the clip, an peak number
FundFreq=[]; %Identified fundamental frequency
PeakFreq=[]; %Measured peak frequencies
PeakEnergy=[]; %Measured energy for each peak
ClosestNoteFreq=[]; %The frequency of the closest identified note
ClosestNoteError=[]; %The error (in cents) of the closest identified note
ClosestNoteName={}; %A string with the identified closest note name

%Step through each clip and build up some per-mode (i.e. for each peak found) lists
for i=1:numel(Results)
    thisResult = Results{i};
    %Step through each identified peak and build up some lists.
    for j=1:numel(thisResult.f_pks)
        ClipNum=[ClipNum i];
        PeakNum=[PeakNum j];
        FundFreq=[FundFreq thisResult.fund_freq];
        PeakFreq = [PeakFreq thisResult.f_pks(j)];
        PeakEnergy = [PeakEnergy thisResult.E_pks(j)];
        ClosestNoteFreq = [ClosestNoteFreq thisResult.closest_note_freq];
        ClosestNoteError = [ClosestNoteError thisResult.closest_note_cents_diff];
        ClosestNoteName = [ClosestNoteName thisResult.closest_note_name];
    end
end

%Want all of the data sorted by note, so the subsequent plots will be
%monotonic increasing
[~, ix] = sort(ClosestNoteFreq);
ClipNum=ClipNum(ix);
PeakNum=PeakNum(ix); 
FundFreq=FundFreq(ix);
PeakFreq=PeakFreq(ix);
PeakEnergy=PeakEnergy(ix);
ClosestNoteFreq=ClosestNoteFreq(ix);
ClosestNoteError=ClosestNoteError(ix); 
ClosestNoteName=ClosestNoteName(ix); 

%Compute the ratios, relative to the _measured_ fundamental
Ratio = PeakFreq./FundFreq;

%Want data for each note plotted in a column, so identify the unique notes,
%and then assign an incrementing index number to each note.
[~,UniqueIncrIdx,NoteNum]=unique(ClosestNoteFreq); %NoteNum contains a unique integer for each note that

%Create a plot of ratios versus note name.
%Scale the symbol size from 0-100 based on log energy
LogMin = min(log(PeakEnergy)); LogMax = max(log(PeakEnergy));
SymSize = 99*(log(PeakEnergy)-LogMin)/(LogMax-LogMin) + 1;
figure;
scatter(NoteNum,Ratio,SymSize);
ylim([0,7]); 
xlabel('Note'); ylabel('Ratio (relative to fundamental)');
set(gca,'Xtick',1:numel(UniqueIncrIdx))
set(gca,'XtickLabel',ClosestNoteName(UniqueIncrIdx))
title('Partial Ratios');
grid

%Create a plot of the cent error for the measured fundamental
figure;
% scatter(NoteNum,ClosestNoteError,SymSize);
scatter(NoteNum,ClosestNoteError,'MarkerFaceColor','b');
set(gca,'Xtick',1:numel(UniqueIncrIdx))
set(gca,'XtickLabel',ClosestNoteName(UniqueIncrIdx))
xlabel('Note'); ylabel('Error [cents]');
title('Frequency Error of Fundamental (442 Hz ref)');
grid
end




